local addOnName = "CustomItemNotes"

local defaultText = ""

local defaultValues = {CustomTooltips = {}, iconSize = 100, iconColor = {r=255,g=255,b=255,a=1}}
local CINSavedVars = {}

local function EditNoteDialogSetup(dialog, data)
	GetControl(dialog, "DisplayName"):SetText(data.itemName)
	GetControl(dialog, "NoteEdit"):SetText(data.note)
end

local function RegisterCustomDialog()
	local dialogControl = GetControl("CIN_EditNoteDialog")

	ZO_Dialogs_RegisterCustomDialog("CIN_EDIT_NOTE",   
	{
		customControl = dialogControl,
		setup = EditNoteDialogSetup,
		title =
		{
			text = CUSTOM_ITEM_NOTES_EDIT_NOTE,
		},
		buttons =
		{
			[1] =
			{
				control =   GetControl(dialogControl, "Save"),
				text =      SI_SAVE,
				callback =  function(dialog)
								local data = dialog.data
								local note = GetControl(dialog, "NoteEdit"):GetText()

								if(note ~= data.note) then
									data.changedCallback(data.itemName, note)
								end
							end,
			},
		
			[2] =
			{
				control =   GetControl(dialogControl, "Cancel"),
				text =      SI_DIALOG_CANCEL,
			}
		}
	})
end

local function GetLinkData(itemLink, itemType)
	local linkData = nil
	local leadString = "|H0:item:"

	if itemType == ITEMTYPE_POTION and IsItemLinkCrafted(itemLink) then
		local superfluous = leadString..string.rep("%d+:", 20)
		local potionEffect = itemLink:gsub(superfluous.."(%d+)", "%1")

		linkData = potionEffect
	elseif itemType == ITEMTYPE_MASTER_WRIT then
		--
	end

	return linkData
end

local function GetCustomTooltip(itemId, linkData)
	if CINSavedVars.CustomTooltips[linkData] then
		return CINSavedVars.CustomTooltips[linkData]
	elseif CINSavedVars.CustomTooltips[itemId] then
		return CINSavedVars.CustomTooltips[itemId]
	end
end

local function GetSlotIds(inventorySlot)
	local bagId, slotIndex

	if inventorySlot.dataEntry then
		bagId = inventorySlot.dataEntry.data.bagId
		slotIndex = inventorySlot.dataEntry.data.slotIndex
	else
		bagId, slotIndex = ZO_Inventory_GetBagAndIndex(inventorySlot)
	end

	local itemId = GetItemId(bagId, slotIndex)
	local itemLink = GetItemLink(bagId, slotIndex)
	local itemType = GetItemType(bagId, slotIndex)

	return bagId, slotIndex, itemId, itemLink, itemType
end

ZO_CreateStringId("SI_BINDING_NAME_CIN_POPUP", GetString(CUSTOM_ITEM_NOTES_EDIT_NOTE))
local function AddNote(inventorySlot, slotActions)
	local valid = ZO_Inventory_GetBagAndIndex(inventorySlot)
	if not valid then return end

	AddCustomMenuItem(GetString(SI_BINDING_NAME_CIN_POPUP), function()
		local _, _, itemId, itemLink, itemType = GetSlotIds(inventorySlot)
		local linkData = GetLinkData(itemLink, itemType)
		local customTooltip = GetCustomTooltip(itemId, linkData)

		local text = customTooltip or defaultText
		ZO_Dialogs_ShowDialog("CIN_EDIT_NOTE", {itemName = itemLink, note = text,
			changedCallback = function(itemName, note)
				if (not note) or note == "" then
					if linkData then CINSavedVars.CustomTooltips[linkData] = nil end
					if itemId then CINSavedVars.CustomTooltips[itemId] = nil end
				else
					CINSavedVars.CustomTooltips[linkData or itemId] = note
				end
			end})
	end)
end

local function OnPlayerActivated()
	LibCustomMenu:RegisterContextMenu(AddNote, LibCustomMenu.CATEGORY_LATE)

	local function ShowCustomTooltip(text)
		if not text or text == "" then return end

		local control = GetControl("ItemTooltip")
		InitializeTooltip(CINTooltip, control, TOPRIGHT, -10, 0, TOPLEFT)

		--CINTooltip:AddHeaderLine(GetString(CUSTOM_ITEM_NOTES_NOTES), "ZoFontWinH5", 1, TOOLTIP_HEADER_SIDE_LEFT, ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		CINTooltip:AddLine(text, "", ZO_NORMAL_TEXT:UnpackRGB())
	end

	ZO_PreHook("ZO_InventorySlot_OnMouseEnter", function(inventorySlot) 
		if inventorySlot.dataEntry then
			local bagId, slotIndex, itemId, itemLink, itemType = GetSlotIds(inventorySlot)
			local linkData = GetLinkData(itemLink, itemType)
			local customTooltip = GetCustomTooltip(itemId, linkData)

			if customTooltip then
				local text = customTooltip
				ShowCustomTooltip(text)
			end
		end
	end)

	ZO_PreHook("ZO_InventorySlot_OnMouseExit", function(inventorySlot) 
		ClearTooltipImmediately(CINTooltip)
	end)
end
EVENT_MANAGER:RegisterForEvent(addOnName, EVENT_PLAYER_ACTIVATED, OnPlayerActivated)

-----
-- LAM
-----

local addOnDispName = "Custom Item Notes"
local author = "Alianym"
local version = 1.23

local panelData = {
	type = "panel",
	name = addOnDispName,
	displayName = addOnDispName,
	author = author,
	version = version,
}

local optionsTable = {
	{
		type = "header",
		name = GetString(SI_GAMEPAD_OPTIONS_MENU),
		width = "full",
	},
	{
		type = "slider",
		name = GetString(CUSTOM_ITEM_NOTES_ICON_SIZE),
		getFunc = function() return CINSavedVars.iconSize end,
		setFunc = function(value) CINSavedVars.iconSize = value end,
		min = 80,
		max = 160,
		step = 10,
		default = 100,
	},
	{
		type = "colorpicker",
		name = GetString(CUSTOM_ITEM_NOTES_ICON_COLOR),
		getFunc = function() return CINSavedVars.iconColor.r, CINSavedVars.iconColor.g, CINSavedVars.iconColor.b, CINSavedVars.iconColor.a end,
		setFunc = function(r,g,b,a) CINSavedVars.iconColor.r=r CINSavedVars.iconColor.g=g CINSavedVars.iconColor.b=b CINSavedVars.iconColor.a=a end,
		default = {r = 255, g = 255, b = 255, a = 1},
	},
}

-----
-----

local function OnLoad(e, AddOnName)
	if AddOnName ~= addOnName then return end

	RegisterCustomDialog()
	CINSavedVarsOLD = ZO_SavedVars:NewAccountWide("CustomItemNotes", 0.1, nil, defaultValues, GetWorldName())
	CINSavedVars = ZO_SavedVars:NewAccountWide("CustomItemNotesSV", 0.1, nil, defaultValues, GetWorldName())

	if not CINSavedVars.migrated then
		CINSavedVars.CustomTooltips = CINSavedVarsOLD.CustomTooltips
		CINSavedVars.migrated = true
	end

	local LAM = LibAddonMenu2
	LAM:RegisterAddonPanel(addOnName, panelData)
	LAM:RegisterOptionControls(addOnName, optionsTable)

	EVENT_MANAGER:UnregisterForEvent(addOnName, EVENT_ADD_ON_LOADED) 
end
EVENT_MANAGER:RegisterForEvent(addOnName, EVENT_ADD_ON_LOADED, OnLoad) 

----------
----------

function CIN_EditNoteDialog_OnInitialized(self)
	--
end

----------
----------

local function CreateNoteIcon(rowControl)
	if GetControl(rowControl:GetName().."CIN_Icon") then return end

	local CIN_NOTE = WINDOW_MANAGER:CreateControl(rowControl:GetName().."CIN_Icon", rowControl, CT_TEXTURE)

	CIN_NOTE:SetDrawLayer(2)
	CIN_NOTE:SetDimensions(24,30)
	CIN_NOTE:ClearAnchors()
	CIN_NOTE:SetAnchor(TOPLEFT,rowControl:GetNamedChild('Highlight'),TOPLEFT,-4,-5)
	CIN_NOTE:SetTexture("EsoUI/Art/Contacts/social_note_up.dds")
	CIN_NOTE:SetHidden(false)
end

--Modified from the example at https://wiki.esoui.com/How_to_run_your_code_before/after_functions_(PreHook/PostHook)
for _, v in pairs(PLAYER_INVENTORY.inventories) do
	local listView = v.listView
	if listView and listView.dataTypes and listView.dataTypes[1] then
		SecurePostHook(listView.dataTypes[1], "setupCallback", function(rowControl, slot)
			local itemId = GetItemId(slot.bagId, slot.slotIndex)
			local itemLink = GetItemLink(slot.bagId, slot.slotIndex)
			local itemType = GetItemType(slot.bagId, slot.slotIndex)

			local linkData = GetLinkData(itemLink, itemType)
			local customTooltip = GetCustomTooltip(itemId, linkData)
			
			local existingControl = GetControl(rowControl:GetName().."CIN_Icon")
			if existingControl then
				existingControl:SetHidden(true)
			end

			if customTooltip and not existingControl then
				CreateNoteIcon(rowControl)
			elseif customTooltip and existingControl then
				local multiplier = CINSavedVars.iconSize / 100
				local r,g,b,a = CINSavedVars.iconColor.r, CINSavedVars.iconColor.g, CINSavedVars.iconColor.b, CINSavedVars.iconColor.a
				existingControl:SetDimensions(24*multiplier,30*multiplier)
				existingControl:SetColor(r,g,b,a)
				existingControl:SetHidden(false) 
			end
		end)
	end
end

--[[[

ITEMTYPE_POTION
ITEMTYPE_MASTER_WRIT

]]